<?php

namespace App\Models;

use App\Mail\SendTwoFactorVerificationCodeMail;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Mail;

class CompanyAdmin extends Authenticatable
{
    use HasFactory,Notifiable,SoftDeletes;

    protected $table    = 'company_admins';
    /**
     * The attributes that are mass assignable.
     *
     * @var string[]
     */
    protected $appends  = ['full_name'];

    protected $fillable = [
        'role_id',
        'company_id',
        'team_id',

        'first_name',
        'last_name',
        'middle_name',
        'email',
        'password',
        'type',
        'remember_token',

        'profile_image',
        'home_phone',
        'cell_phone',
        'start_date',

        'address',
        'address_2',
        'city',
        'state',
        'zip',
        'latitude',
        'longitude',
        'timezone',
        'timezone_offset',
        'timezone_abbr',
        'address_verified',

        'sms',
        'notify_email',
        'reminder_notify',
        'allow_login',
        'notes',

        'termination_date',
        'termination_reason',
        'termination_rehire',
        'termination_equipment',
        'termination_none',

        'extension_number',
        'text_number',
        'after_hours_extension',
        'left_drawer',
        'per_page_view',

        'delivery_date_option',
        'color_options',
        'time_format',
        'weekly_strat_day',

        'notification',
        'notifictiono_hours',

        'logo',
        'email_logo',
        'campany_address',
        'campany_address2',
        'campany_city',
        'campany_state',
        'campany_zip',
        'billing_information',
        'contact_information',

        'session_lifetime',
        'timezone_id',
        'timezone_type',

        'two_factor_secret',
        'two_factor_expires_at',

        'status',
        'created_at',
        'updated_at',
        'deleted_at'
    ];

    protected $dates    = ['deleted_at'];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array
     */
    protected $hidden   = [
        'password',
        'remember_token',
    ];

    //add new user to chat general board
    public static function boot(){
        parent::boot();

        self::created(function($model){
            chatGeneralBoardCreate($model);
        });
    }

    // Type
    // 0 - Company Admin
    // 1 - Manager
    // 2 - Dispatcher

    /*
     * Scopes
     */
    public function scopeCompany($query){
        return $query->where('company_id',auth()->guard('company-admin')->user()->company_id);
    }

    public function scopeCompanyAdmin($query){
        return $query->where('type',0);
    }

    public function scopeManager($query){
        return $query->where('type',1);
    }

    public function scopeDispatcher($query){
        return $query->where('type',2);
    }


    /*
     * Mutators
     */
    public function setStartDateAttribute($value): ?string {
        return $this->attributes['start_date'] = Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s');
    }


    /*
     * Accessors
     */
    public function getStartDateAttribute($value): ?string {
        return Carbon::parse($value)->format(config('settings.date_format','m/d/Y'));
    }

    public function getFullNameAttribute($value): ?string {
        return $this->attributes['first_name'].' '.$this->attributes['last_name'];
    }

    /*
     * Relationships
     */
    public function team(){
        return $this->belongsTo(Team::class);
    }

    public function getCompany(){
        return $this->belongsTo(Company::class,'company_id','id');
    }

    public function dispatcher_active_drivers(){
        return $this->hasMany(Driver::class, 'manager_id', 'id');
    }

    public function dispatcher_inactive_drivers(){
        return $this->hasMany(Driver::class, 'manager_id', 'id')->onlyTrashed();
    }

    public function notificationDaytime(){
        return $this->hasMany(NotificationDaytime::class,'user_id','id')->whereIn('user_type',['0','1','2'])->orderBy('id','asc');
    }

    public function getTimezone(){
        return $this->hasOne(Timezone::class,'id','timezone_id');
    }

    public function generateTwoFactorCode() {
        try {
            $twoFactorSettings = TwoFactorSetting::IsCompanyAdmin()->first();

            $this->timestamps = false;
            $this->two_factor_secret = rand(100000, 999999);
            $this->two_factor_expires_at = now()->addMinutes(10);
            $this->save();

            if($twoFactorSettings->enable_email_verify == '1' && !is_null($this->email)){
                $data = [
                    'name' => $this->full_name,
                    'code' => $this->two_factor_secret,
                ];

                //Mail::to($this->email)->send(new SendTwoFactorVerificationCodeMail($data));
                sendEmail($this->email, new SendTwoFactorVerificationCodeMail($data));
            }
            if($twoFactorSettings->enable_sms_verify == '1' && !is_null($this->cell_phone) && $this->sms == 1){
                // Twilio's credentials are fetched from env File
                sendSMS($this->cell_phone, 'Please use this otp for 2-factor verification: '.$this->two_factor_secret);
            }
        } catch (Exception $e) {
            info("Error: ". $e->getMessage());
        }
    }
}
